package com.fzm.admin.listener;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.fzm.common.config.RabbitMQConfig;
import com.fzm.common.entity.dto.CopyrightQueryRequest;
import com.fzm.common.entity.dto.CopyrightQueryResponse;
import com.fzm.common.entity.dto.CopyrightResponse;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.properties.CopyrightProperties;
import com.fzm.common.utils.CopyrightSignUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.TreeMap;

/**
 * @author tangtuo
 * @date 2021/12/23 14:56
 */
@Slf4j
@Component
public class CopyrightLister {

    @Resource
    private CopyrightProperties copyrightProperties;

    /**
     * 主动查询当前流水号对应的版权审批结果
     *
     * @param serial_code
     */
    @RabbitListener(queues = RabbitMQConfig.COPYRIGHT_QUEUE)
    public void listerCopyrightApply(String serial_code) {
        log.info("监听者接收消息, 流水号为: {}", serial_code);
        CopyrightQueryRequest request = new CopyrightQueryRequest(serial_code, copyrightProperties.getAppId());
        TreeMap<String, String> map = this.beanToMap(request, copyrightProperties.getAppSecret());
        String sign = CopyrightSignUtil.generateSign(map);
        request.setSign(sign);
        String json = JSONUtil.toJsonStr(request);
        log.info("查询审核结果, 当前流水号为:{} , 请求参数为 : {}", serial_code, json);
        String response = HttpUtil.post(copyrightProperties.getQueryUrl(), json, 10000);
        log.info("查询审核结果接口响应内容: {}", response);
        if (StringUtils.isBlank(response)) {
            throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, "版权局接口无响应,请稍后重试");
        }
        CopyrightQueryResponse copyrightResponse = JSONUtil.toBean(response, CopyrightQueryResponse.class);
        if (copyrightResponse == null) {
            throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, "版权局接口无响应,请稍后重试");
        }
        if (!copyrightResponse.isSuccess()) {
            throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, copyrightResponse.getMessage());
        }
        // 如果审核结果还是审核中的话,就继续把流水号丢进延时队列,2小时后再查一次

        // 审核成功或失败的话,更新审核结果
    }

    private TreeMap<String, String> beanToMap(CopyrightQueryRequest request, String app_secret) {
        TreeMap<String, String> map = JSONUtil.toBean(JSONUtil.toJsonStr(request), new TypeReference<TreeMap<String, String>>() {
        }, true);
        map.put("biz_content", JSONUtil.toJsonStr(request.getBiz_content()));
        map.put("app_secret", app_secret);
        return map;
    }
}
